// udStFiLrDelimitedText
// J. J. Weimer
// History: $Revision: 806 $ on $Date: 2008-07-07 11:50:22 -0500 (Mon, 07 Jul 2008) $
//
// Procedure files to read a tab delimited file and store it
//
// Access to Package
//
// -> udStFiLrDelimitedText(udFL)
//		STRUCTURE udFLStandardStructure &udFL
//
// see the structure definition and User's Manual

#pragma rtGlobals=1
#pragma IgorVersion=6.03
#pragma version=1.2

#include "udFLStandardFunctions", versions>=2.0

// **********************************************
// Globals and Localizations

Static Constant kVersion = 1.2

// **********************************************
// Initialize (Zero + Query + Check for PlugIns)

Static Function udFLDelimitedTextInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLDelimitedTextQuery(udFL)
	return 0
end

// **********************************************
// Query

Static Function udFLDelimitedTextQuery(udFL)
	STRUCT udFLStandardStructure &udFL
		
	udFL.mimetype = "TEXT"
	udFL.extensions = ".txt;.dat;"
	udFL.loadModes += "Manual;"
	udFL.procModes += "(X Scale) + Y's;(X Scale, Y) Pairs;"
	udFL.itemsType = 4			// returns waves
	udFL.itemsDim[p] = p*0
	udFL.itemsDim[0]=-1			// waves are nominally unscaled numeric
	udFL.setNames=(2+8+32)		// filename+number or prefix+number or loader-defined supported
	udFL.userData = "Prefix+Column#;"
	udFLDelimitedTextInitPlugIns(udFL)
	return 0	
end

// **********************************************
// Check for PlugIns

Static Function udFLDelimitedTextInitPlugIns(udFL)
	STRUCT udFLStandardStructure &udFL

	udFLCheckForPlugIn("udFLPlugInLinkDisplay",udFL)
	return 0
end

// **********************************************
// File Loader

Function udStFiLrDelimitedText(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum, ic, nt
	string pathStr, wavesLoaded, twList
	
	switch(udFL.userLCtrl)
		case -1:		// initialize		
			udFLDelimitedTextInititalize(udFL)		
			return 0
		case 0:		// query
			udFLDelimitedTextQuery(udFL)
			return 0
		case 1:		// auto
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,"No files in the selection list!",-1)
				return -1
			endif
			break
		case 2:		// manual
			Open/C="TEXT"/D/R/M="Select the tab-delimited file" refNum
			if (strlen(S_filename)!=0)
				udFL.filelist = S_filename + ";"
			else
				udFLSetError(udFL,"No file selected",-1)
				return -1
			endif
			break
	endswitch
	
	if (udFL.reportCtrl >= 1)
		print "LOADING DELIMITED DATA"
		print "... assuring file paths"
	endif
	if (udFLAssureFilePaths(udFL)<0)
		return -1
	endif
	
	if (udFL.reportCtrl >= 1)
		print "... setting data folder location"
	endif
	string cdf = GetDataFolder(1)
	udFLSetDataFolder(udFL)
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr
		NewPath/O/Q tmpDelimitedTextdir, pathStr
	endif
	
	if (udFL.reportCtrl >= 1)
		print "... loading"
	endif
		
	nt = ItemsInList(udFL.fileList)
	for (ic=0;ic<nt;ic+=1)
			
		WavesLoaded = LoadDelimitedTextFile(ic,udFL)
		
		if (strlen(WavesLoaded)!=0)
				
			WavesLoaded = ProcessDelimitedTextWaves(WavesLoaded, udFL)
			WavesLoaded = RenameDelimitedTextWaves(ic,WavesLoaded, udFL)
			WavesLoaded = DoUserDataDelimitedTextWaves(WavesLoaded, udFL)
			LinkDelimitedTextWaves(WavesLoaded,udFL)
			udFL.itLst.WAVES += WavesLoaded
				
		endif
		
	endfor
	
	udFL.itemsCount = ItemsInList(udFL.itLst.WAVES)
	
	SetDataFolder $cdf
	
	if (udFL.reportCtrl == 2)
		print "Structure @ Loader:", udFL
	endif
	
	return 0
end

// **********************************************
// Load A File

Static Function/S LoadDelimitedTextFile(ic,udFL)
	variable ic
	STRUCT udFLStandardStructure &udFL
	
	string fileStr=StringFromList(ic,udFL.fileList)
	if (udFL.reportCtrl >= 1)
		print "... file ", fileStr
	endif
	if (udFL.reportCtrl < 0)
		if (strlen(udFL.pathStr)==0)
			LoadWave/Q/N=TDwave/J/K=0 fileStr
		else
			LoadWave/Q/N=TDwave/J/K=0/P=tmpDelimitedTextdir fileStr
		endif
	else
		if (strlen(udFL.pathStr)==0)
			LoadWave/A/J/K=0 fileStr
		else
			LoadWave/A/J/K=0/P=tmpDelimitedTextdir fileStr
		endif
	endif
	
	return S_wavenames
end

// **********************************************
// Process the Waves

Static Function/S ProcessDelimitedTextWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL
	
	variable xs=0, xe=0, ni=0, jc=0
	string xwave, ywave, wList = "", theStr = "", itHas = ""
		
	switch((udFL.userPCtrl)-1)
		case 1:	// (X scale) + Y's
			ni = ItemsInList(WavesLoaded)
			if (ni<2)
				udFLSetError(udFL,"Could not scale only one wave",0)
				break
			endif
			if (udFL.reportCtrl >= 1)
				print "... scaling all wave(s) based on data of equal increment in first column"
			endif
			xwave = StringFromList(0,WavesLoaded)
			wave nwave=$xwave
			xs = nwave[0]
			xe = nwave[numpnts(nwave)-1]
			killwaves/Z nwave
			for (jc=1;jc<ni;jc+=1)
				ywave = StringFromList(jc,WavesLoaded)
				wave nwave = $ywave
				SetScale/I x, xs, xe, nwave
				wList += ywave + ";"
			endfor
			WavesLoaded = wList
			break
		case 2:	// (X Scale, Y) Pairs
			ni = ItemsInList(WavesLoaded)
			if (ni<2)
				udFLSetError(udFL,"Could not scale only one wave",0)
				break
			endif
			if (mod(ni,2)!=0)
				udFLSetError(udFL,"Could not scale as wave pairs with odd number of columns",0)
				break
			endif
			if (udFL.reportCtrl >= 1)
				print "... scaling wave(s) as paired (x,y) columns"
			endif
			for (jc=0;jc<ni;jc+=2)
				xwave = StringFromList(jc,WavesLoaded)
				wave nwave=$xwave
				xs = nwave[0]
				xe = nwave[numpnts(nwave)-1]
				killwaves/Z nwave
				ywave = StringFromList(jc+1,WavesLoaded)
				wave nwave = $ywave
				SetScale/I x, xs, xe, nwave
				wList += ywave + ";"
			endfor
			WavesLoaded = wList
			break
	endswitch
	return WavesLoaded
end

// **********************************************
// Apply User Data to Waves
//
// [...] wave selection

Static Function/S DoUserDataDelimitedTextWaves(WavesLoaded, udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL	

	string theStr = "", itHas = "", xwave
	variable ni, xe, xs, jc
	
	theStr = udFLStripUserDataString(udFL)
	if (strlen(theStr)==0)
		return WavesLoaded
	endif

	SplitString/E="\[(.+)\]" theStr, itHas	
	if (strlen(itHas)==0)
		return WavesLoaded
	endif
	
	ni = ItemsInList(WavesLoaded)
	if (ni>2)
		if (udFL.reportCtrl >= 1)
			print "... selecting wave(s) based on input: ", itHas
		endif
		theStr = ""
		xe = ItemsInList(itHas,",")
		for (jc=0;jc<xe;jc+=1)
			theStr += ExpandString(StringFromList(jc,itHas,","),sep=";")
		endfor
		itHas = ""
		xs = 0
		xe = str2num(StringFromList(xs,theStr))
		for(jc=1;jc<(ni+1);jc+=1)
			if (jc != xe)
				xwave = StringFromList(jc-1,WavesLoaded)
				wave nwave = $xwave
				killwaves/Z nwave
			else
				itHas += StringFromList(jc-1,WavesLoaded) + ";"
				xs += 1
				xe = str2num(StringFromList(xs,theStr))
			endif
		endfor
		WavesLoaded = itHas
	else
		udFLSetError(udFL,"Cannot select with only one wave input",0)
	endif
	return WavesLoaded
end

// **********************************************
// Rename Waves

Static Function/S RenameDelimitedTextWaves(ic,WavesLoaded,udFL)
	variable ic
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	string theStr, itHas, columnList, fileStr = StringFromList(ic,udFL.fileList)
	variable jc, ni
		
	switch(udFL.setNames)
		case 1:
			WavesLoaded=udFLRenameInput(WavesLoaded,"TDwave")
			break
		case 2:
			udFL.namePrefix = ReplaceString(" ",ParseFilePath(3,fileStr,":",0,0),"")
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by filename ", udFL.namePrefix, " + number"
			endif
			WavesLoaded=udFLRenameInput(WavesLoaded,udFL.namePrefix)
			break
		case 8:
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by prefix ", udFL.namePrefix, " + number"
			endif
			WavesLoaded=udFLRenameInput(WavesLoaded,udFL.namePrefix)
			break
		case 32:
			if (udFL.reportCtrl >= 1)
				print "... renaming wave(s) by prefix ", udFL.namePrefix, " + column number"
			endif
			columnList = WavesLoaded
			ni = ItemsInList(columnList)
			WavesLoaded = ""
			for (jc=0;jc<ni;jc+=1)
				theStr = StringFromList(jc,columnList)
				WavesLoaded+=udFLRenameInput(theStr,udFL.namePrefix,index=(jc+1))
			endfor
			break
	endswitch
	return WavesLoaded
end

// **********************************************
// (possibly) Link the Waves

Static Function LinkDelimitedTextWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	string wList = udFL.itLst.WAVES

#if(exists("udFLPlugInLinkDisplay")==6)
	udFL.userPCtrl+=-3
	udFL.itLst.WAVES=WavesLoaded
	udFLPlugInLinkDisplay(udFL)
	udFL.itLst.WAVES=wList
	udFL.userPCtrl+=3
#endif

	return 0
end

// **********************************************
// Support Functions

Static Function/S ExpandString(theStr, [sep])
	string theStr, sep
	
	string xs, xe
	variable ns, ne, ic

	if(ParamIsDefault(sep))
		sep = ","
	endif
	
	SplitString/E="(.+)-(.+)" theStr, xs,xe
	
	if (strlen(xs)==0)
		return (theStr + sep)
	endif
	ns = str2num(xs)
	ne = str2num(xe)
	
	theStr = ""
	for(ic=ns;ic<(ne+1);ic+=1)
		theStr += num2str(ic) + sep
	endfor
	return theStr
end
